@@ -57,8 +57,8 @@ API Reference
 #include "sqlite3.h"
 #endif
 
-#if SQLITE_VERSION_NUMBER < 3007000
-#error Your SQLite version is too old.  It must be at least 3.7.0
+#if SQLITE_VERSION_NUMBER < 3007006
+#error Your SQLite version is too old.  It must be at least 3.7.6
 #endif
 
 /* system headers */
@@ -113,7 +113,10 @@ static int APSW_Should_Fault(const char *);
 static PyObject *apswmodule;
 
 /* Everything except the module itself is in seperate files */
-
+#ifdef PYPY_VERSION
+#include "pypycompat.c"
+#endif
+ 
 /* Augment tracebacks */
 #include "traceback.c"
 
@@ -310,10 +313,8 @@ config(APSW_ARGUNUSED PyObject *self, PyObject *args)
   long opt;
 
   if(PyTuple_GET_SIZE(args)<1 || !PyIntLong_Check(PyTuple_GET_ITEM(args, 0)))
-    {
-      PyErr_Format(PyExc_TypeError, "There should be at least one argument with the first being a number");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "There should be at least one argument with the first being a number");
+
   opt=PyIntLong_AsLong(PyTuple_GET_ITEM(args,0));
   if(PyErr_Occurred())
     return NULL;
@@ -352,8 +353,7 @@ config(APSW_ARGUNUSED PyObject *self, PyObject *args)
 	  }
 	else if(!PyCallable_Check(logger))
 	  {
-	    PyErr_Format(PyExc_TypeError, "Logger should be None or a callable");
-	    return NULL;
+	    return PyErr_Format(PyExc_TypeError, "Logger should be None or a callable");
 	  }
 	else
 	  {
@@ -369,8 +369,7 @@ config(APSW_ARGUNUSED PyObject *self, PyObject *args)
       }
 
     default:
-      PyErr_Format(PyExc_TypeError, "Unknown config type %d", (int)opt);
-      return NULL;
+      return PyErr_Format(PyExc_TypeError, "Unknown config type %d", (int)opt);
     }
 
   SET_EXC(res, NULL);
@@ -421,23 +420,24 @@ memoryhighwater(APSW_ARGUNUSED PyObject *self, PyObject *args)
 }
 
 
-/** .. method:: softheaplimit(bytes)
+/** .. method:: softheaplimit(bytes) -> oldlimit
 
-  Requests SQLite try to keep memory usage below *bytes* bytes.
+  Requests SQLite try to keep memory usage below *bytes* bytes and
+  returns the previous setting.
 
-  -* sqlite3_soft_heap_limit
+  -* sqlite3_soft_heap_limit64
 */
 static PyObject*
 softheaplimit(APSW_ARGUNUSED PyObject *self, PyObject *args)
 {
-  int limit;
+  long long limit, oldlimit;
 
-  if(!PyArg_ParseTuple(args, "i", &limit))
+  if(!PyArg_ParseTuple(args, "L", &limit))
     return NULL;
 
-  sqlite3_soft_heap_limit(limit);
+  oldlimit=sqlite3_soft_heap_limit64(limit);
 
-  Py_RETURN_NONE;
+  return PyLong_FromLongLong(oldlimit);
 }
 
 /** .. method:: randomness(bytes)  -> data
@@ -458,10 +458,7 @@ randomness(APSW_ARGUNUSED PyObject *self, PyObject *args)
   if(!PyArg_ParseTuple(args, "i", &amount))
     return NULL;
   if(amount<0)
-    {
-      PyErr_Format(PyExc_ValueError, "Can't have negative number of bytes");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_ValueError, "Can't have negative number of bytes");
   bytes=PyBytes_FromStringAndSize(NULL, amount);
   if(!bytes) return bytes;
   sqlite3_randomness(amount, PyBytes_AS_STRING(bytes));
@@ -571,10 +568,7 @@ getapswexceptionfor(APSW_ARGUNUSED PyObject *self, PyObject *pycode)
   PyObject *result=NULL;
 
   if(!PyIntLong_Check(pycode))
-    {
-      PyErr_Format(PyExc_TypeError, "Argument should be an integer");
-      return NULL;
-    }
+      return PyErr_Format(PyExc_TypeError, "Argument should be an integer");
   code=PyIntLong_AsLong(pycode);
   if(PyErr_Occurred()) return NULL;
 
@@ -586,10 +580,7 @@ getapswexceptionfor(APSW_ARGUNUSED PyObject *self, PyObject *pycode)
         break;
       }
   if(!result)
-    {
-      PyErr_Format(PyExc_ValueError, "%d is not a known error code", code);
-      return result;
-    }
+    return PyErr_Format(PyExc_ValueError, "%d is not a known error code", code);
 
   PyObject_SetAttrString(result, "extendedresult", PyInt_FromLong(code));
   PyObject_SetAttrString(result, "result", PyInt_FromLong(code&0xff));
@@ -792,15 +783,10 @@ apsw_async_control(APSW_ARGUNUSED PyObject *self, PyObject *args)
   int op, inparam, outparam,res;
 
   if(!PyTuple_Check(args) || PyTuple_GET_SIZE(args)<1)
-    {
-      PyErr_Format(PyExc_TypeError, "Args should be a tuple of at least one item");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "Args should be a tuple of at least one item");
+
   if(!PyIntLong_Check(PyTuple_GET_ITEM(args, 0)))
-    {
-      PyErr_Format(PyExc_TypeError, "Arg 0 must be a number");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "Arg 0 must be a number");
 
   switch(PyIntLong_AsLong(PyTuple_GET_ITEM(args, 0)))
     {
@@ -832,8 +818,7 @@ apsw_async_control(APSW_ARGUNUSED PyObject *self, PyObject *args)
       return PyInt_FromLong(outparam);
 
     default:
-      PyErr_Format(PyExc_ValueError, "Unknown operation argument");
-      return NULL;
+      return PyErr_Format(PyExc_ValueError, "Unknown operation argument");
     }
 }
 
@@ -1177,10 +1162,7 @@ formatsqlvalue(APSW_ARGUNUSED PyObject *self, PyObject *value)
 #if PY_MAJOR_VERSION<3
   /* We don't support plain strings only unicode */
   if(PyString_Check(value))
-    {
-      PyErr_Format(PyExc_TypeError, "Old plain strings not supported - use unicode");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "Old plain strings not supported - use unicode");
 #endif
   /* Unicode */
   if(PyUnicode_Check(value))
@@ -1272,8 +1254,7 @@ formatsqlvalue(APSW_ARGUNUSED PyObject *self, PyObject *value)
       return unires;
     }
 
-  PyErr_Format(PyExc_TypeError, "Unsupported type");
-  return NULL;
+  return PyErr_Format(PyExc_TypeError, "Unsupported type");
 }
 
 /** .. automethod:: main()
@@ -1451,6 +1432,8 @@ PyInit_apsw(void)
 #endif
 
     if (m == NULL)  goto fail;
+    
+    Py_INCREF(m);
 
     if(init_exceptions(m)) goto fail;
 
@@ -1723,6 +1706,8 @@ modules etc. For example::
 
       DICT("mapping_db_config"),
       ADDINT(SQLITE_DBCONFIG_LOOKASIDE),
+      ADDINT(SQLITE_DBCONFIG_ENABLE_FKEY),
+      ADDINT(SQLITE_DBCONFIG_ENABLE_TRIGGER),
       END,
 
       DICT("mapping_status"),
@@ -1735,12 +1720,18 @@ modules etc. For example::
       ADDINT(SQLITE_STATUS_PARSER_STACK),
       ADDINT(SQLITE_STATUS_PAGECACHE_SIZE),
       ADDINT(SQLITE_STATUS_SCRATCH_SIZE),
+      ADDINT(SQLITE_STATUS_MALLOC_COUNT),
       END,
 
       DICT("mapping_db_status"),
       ADDINT(SQLITE_DBSTATUS_LOOKASIDE_USED),
       ADDINT(SQLITE_DBSTATUS_CACHE_USED),
       ADDINT(SQLITE_DBSTATUS_MAX),
+      ADDINT(SQLITE_DBSTATUS_SCHEMA_USED),
+      ADDINT(SQLITE_DBSTATUS_STMT_USED),
+      ADDINT(SQLITE_DBSTATUS_LOOKASIDE_HIT),
+      ADDINT(SQLITE_DBSTATUS_LOOKASIDE_MISS_FULL),
+      ADDINT(SQLITE_DBSTATUS_LOOKASIDE_MISS_SIZE),
       END,
 
       DICT("mapping_locking_level"),
@@ -1778,12 +1769,21 @@ modules etc. For example::
       ADDINT(SQLITE_SYNC_DATAONLY),
       END,
 
+      DICT("mapping_wal_checkpoint"),
+      ADDINT(SQLITE_CHECKPOINT_PASSIVE),
+      ADDINT(SQLITE_CHECKPOINT_FULL),
+      ADDINT(SQLITE_CHECKPOINT_RESTART),
+      END,
+
       DICT("mapping_file_control"),
       ADDINT(SQLITE_FCNTL_LOCKSTATE),
       ADDINT(SQLITE_GET_LOCKPROXYFILE),
       ADDINT(SQLITE_SET_LOCKPROXYFILE),
       ADDINT(SQLITE_LAST_ERRNO),
       ADDINT(SQLITE_FCNTL_SIZE_HINT),
+      ADDINT(SQLITE_FCNTL_CHUNK_SIZE),
+      ADDINT(SQLITE_FCNTL_FILE_POINTER),
+      ADDINT(SQLITE_FCNTL_SYNC_OMITTED),
       END
 
 #ifdef APSW_USE_SQLITE_ASYNCVFS_H
@@ -1871,6 +1871,7 @@ modules etc. For example::
 static void
 add_shell(PyObject *apswmodule)
 {
+#ifndef PYPY_VERSION
   PyObject *res=NULL, *maindict=NULL, *apswdict, *msvciscrap=NULL;
 
   maindict=PyModule_GetDict(PyImport_AddModule("__main__"));
@@ -1893,6 +1894,7 @@ add_shell(PyObject *apswmodule)
   assert(res);
   Py_XDECREF(res);
   Py_XDECREF(msvciscrap);
+#endif
 }
 
 #ifdef APSW_TESTFIXTURES
