@@ -26,7 +26,7 @@ processes.
 /* details of a registered function passed as user data to sqlite3_create_function */
 typedef struct FunctionCBInfo {
   PyObject_HEAD
-  char *name;                     /* ascii function name */
+  char *name;                     /* utf8 function name */
   PyObject *scalarfunc;           /* the function to call for stepping */
   PyObject *aggregatefactory;     /* factory for aggregate functions */
 } FunctionCBInfo;
@@ -50,7 +50,6 @@ struct Connection {
 
   PyObject *dependents;           /* tracking cursors & blobs belonging to this connection */
   PyObject *dependent_remove;     /* dependents.remove for weak ref processing */
-  PyObject *functions;            /* list of registered functions */
 
   /* registered hooks/handlers (NULL or callable) */
   PyObject *busyhandler;     
@@ -134,7 +133,6 @@ FunctionCBInfo_dealloc(FunctionCBInfo *self)
 static void
 Connection_internal_cleanup(Connection *self)
 {
-  Py_CLEAR(self->functions);
   Py_CLEAR(self->busyhandler);
   Py_CLEAR(self->rollbackhook);
   Py_CLEAR(self->profile);
@@ -324,7 +322,6 @@ Connection_new(PyTypeObject *type, APSW_ARGUNUSED PyObject *args, APSW_ARGUNUSED
       self->dependents=PyList_New(0);
       self->dependent_remove=PyObject_GetAttrString(self->dependents, "remove");
       self->stmtcache=0;
-      self->functions=PyList_New(0);
       self->busyhandler=0;
       self->rollbackhook=0;
       self->profile=0;
@@ -933,10 +930,7 @@ Connection_setupdatehook(Connection *self, PyObject *callable)
     }
 
   if(!PyCallable_Check(callable))
-    {
-      PyErr_Format(PyExc_TypeError, "update hook must be callable");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "update hook must be callable");
 
   PYSQLITE_VOID_CALL(sqlite3_update_hook(self->db, updatecb, self));
 
@@ -1003,10 +997,7 @@ Connection_setrollbackhook(Connection *self, PyObject *callable)
     }
 
   if(!PyCallable_Check(callable))
-    {
-      PyErr_Format(PyExc_TypeError, "rollback hook must be callable");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "rollback hook must be callable");
 
   PYSQLITE_VOID_CALL(sqlite3_rollback_hook(self->db, rollbackhookcb, self));
 
@@ -1075,10 +1066,7 @@ Connection_setprofile(Connection *self, PyObject *callable)
     }
 
   if(!PyCallable_Check(callable))
-    {
-      PyErr_Format(PyExc_TypeError, "profile function must be callable");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "profile function must be callable");
 
   PYSQLITE_VOID_CALL(sqlite3_profile(self->db, profilecb, self));
 
@@ -1165,10 +1153,7 @@ Connection_setcommithook(Connection *self, PyObject *callable)
     }
 
   if(!PyCallable_Check(callable))
-    {
-      PyErr_Format(PyExc_TypeError, "commit hook must be callable");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "commit hook must be callable");
 
   PYSQLITE_VOID_CALL(sqlite3_commit_hook(self->db, commithookcb, self));
 
@@ -1255,10 +1240,7 @@ Connection_setwalhook(Connection *self, PyObject *callable)
     }
   
   if(!PyCallable_Check(callable))
-  {
-      PyErr_Format(PyExc_TypeError, "wal hook must be callable");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "wal hook must be callable");
 
   PYSQLITE_VOID_CALL(sqlite3_wal_hook(self->db, walhookcb, self));
 
@@ -1344,10 +1326,7 @@ Connection_setprogresshandler(Connection *self, PyObject *args)
     }
 
   if(!PyCallable_Check(callable))
-    {
-      PyErr_Format(PyExc_TypeError, "progress handler must be callable");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "progress handler must be callable");
 
   PYSQLITE_VOID_CALL(sqlite3_progress_handler(self->db, nsteps, progresshandlercb, self));
   Py_INCREF(callable);
@@ -1465,10 +1444,7 @@ Connection_setauthorizer(Connection *self, PyObject *callable)
     }
 
   if(!PyCallable_Check(callable))
-    {
-      PyErr_Format(PyExc_TypeError, "authorizer must be callable");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "authorizer must be callable");
 
   APSW_FAULT_INJECT(SetAuthorizerFail,
                     PYSQLITE_CON_CALL(res=sqlite3_set_authorizer(self->db, authorizercb, self)),
@@ -1556,10 +1532,7 @@ Connection_collationneeded(Connection *self, PyObject *callable)
     }
 
   if(!PyCallable_Check(callable))
-    {
-      PyErr_Format(PyExc_TypeError, "collationneeded callback must be callable");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "collationneeded callback must be callable");
 
   APSW_FAULT_INJECT(CollationNeededFail,
                     PYSQLITE_CON_CALL(res=sqlite3_collation_needed(self->db, self, collationneeded_cb)), 
@@ -1659,10 +1632,7 @@ Connection_setbusyhandler(Connection *self, PyObject *callable)
     }
 
   if(!PyCallable_Check(callable))
-    {
-      PyErr_Format(PyExc_TypeError, "busyhandler must be callable");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "busyhandler must be callable");
 
   APSW_FAULT_INJECT(SetBusyHandlerFail,
                     PYSQLITE_CON_CALL(res=sqlite3_busy_handler(self->db, busyhandlercb, self)),
@@ -2245,6 +2215,16 @@ cbdispatch_final(sqlite3_context *context)
   PyGILState_Release(gilstate);
 }
 
+/* Used for the create function v2 xDestroy callbacks.  Note this is
+   called even when supplying NULL for the function implementation (ie
+   deleting it), so XDECREF has to be used.
+ */
+static void
+apsw_free_func(void *funcinfo)
+{
+  Py_XDECREF((PyObject*)funcinfo);
+}
+
 /** .. method:: createscalarfunction(name, callable[, numargs=-1])
 
   Registers a scalar function.  Scalar functions operate on one set of paramaters once.  
@@ -2270,7 +2250,7 @@ cbdispatch_final(sqlite3_context *context)
      * :ref:`Example <scalar-example>`
      * :meth:`~Connection.createaggregatefunction`
 
-  -* sqlite3_create_function
+  -* sqlite3_create_function_v2
 */
 
 static PyObject *
@@ -2279,7 +2259,6 @@ Connection_createscalarfunction(Connection *self, PyObject *args)
   int numargs=-1;
   PyObject *callable;
   char *name=0;
-  char *chk;
   FunctionCBInfo *cbinfo;
   int res;
  
@@ -2292,23 +2271,6 @@ Connection_createscalarfunction(Connection *self, PyObject *args)
   assert(name);
   assert(callable);
 
-  /* there isn't a C api to get a (potentially unicode) string and
-     make it uppercase so we hack around  */
-
-  /* validate the name */
-  for(chk=name;*chk && !((*chk)&0x80);chk++);
-  if(*chk)
-    {
-      PyMem_Free(name);
-      PyErr_SetString(PyExc_TypeError, "function name must be ascii characters only");
-      return NULL;
-    }
-
-  /* convert name to upper case */
-  for(chk=name;*chk;chk++)
-    if(*chk>='a' && *chk<='z')
-      *chk-='a'-'A';
-
   if(callable!=Py_None && !PyCallable_Check(callable))
     {
       PyMem_Free(name);
@@ -2330,31 +2292,28 @@ Connection_createscalarfunction(Connection *self, PyObject *args)
     }
 
   PYSQLITE_CON_CALL(
-                res=sqlite3_create_function(self->db,
-                                            name,
-                                            numargs,
-                                            SQLITE_UTF8,
-                                            cbinfo,
-                                            cbinfo?cbdispatch_func:NULL,
-                                            NULL,
-                                            NULL)
+                res=sqlite3_create_function_v2(self->db,
+					       name,
+					       numargs,
+					       SQLITE_UTF8,
+					       cbinfo,
+					       cbinfo?cbdispatch_func:NULL,
+					       NULL,
+					       NULL,
+					       apsw_free_func)
                 );
-  if(callable==Py_None)
-    PyMem_Free(name);
-
   if(res)
     {
+      /* Note: On error sqlite3_create_function_v2 calls the
+	 destructor (apsw_free_func)! */
       SET_EXC(res, self->db);
       goto finally;
     }
 
-  if(cbinfo)
-    PyList_Append(self->functions, (PyObject*)cbinfo);
-  
+  if(callable==Py_None)
+    PyMem_Free(name);
+
  finally:
-  /* cbinfo will be copied into list on success else we need to dump
-     it anyway */
-  Py_XDECREF(cbinfo);
   if(PyErr_Occurred())
     return NULL;
   Py_RETURN_NONE;
@@ -2397,7 +2356,7 @@ Connection_createscalarfunction(Connection *self, PyObject *args)
      * :ref:`Example <aggregate-example>`
      * :meth:`~Connection.createscalarfunction`
 
-  -* sqlite3_create_function
+  -* sqlite3_create_function_v2
 */
 
 static PyObject *
@@ -2406,7 +2365,6 @@ Connection_createaggregatefunction(Connection *self, PyObject *args)
   int numargs=-1;
   PyObject *callable;
   char *name=0;
-  char *chk;
   FunctionCBInfo *cbinfo;
   int res;
 
@@ -2419,22 +2377,6 @@ Connection_createaggregatefunction(Connection *self, PyObject *args)
   assert(name);
   assert(callable);
 
-  /* there isn't a C api to get a (potentially unicode) string and make it uppercase so we hack around  */
-
-  /* validate the name */
-  for(chk=name;*chk && !((*chk)&0x80);chk++);
-  if(*chk)
-    {
-      PyMem_Free(name);
-      PyErr_SetString(PyExc_TypeError, "function name must be ascii characters only");
-      return NULL;
-    }
-
-  /* convert name to upper case */
-  for(chk=name;*chk;chk++)
-    if(*chk>='a' && *chk<='z')
-      *chk-='a'-'A';
-
   if(callable!=Py_None && !PyCallable_Check(callable))
     {
       PyMem_Free(name);
@@ -2455,32 +2397,29 @@ Connection_createaggregatefunction(Connection *self, PyObject *args)
     }
 
   PYSQLITE_CON_CALL(
-                res=sqlite3_create_function(self->db,
-                                            name,
-                                            numargs,
-                                            SQLITE_UTF8,  /* it isn't very clear what this parameter does */
-                                            cbinfo,
-                                            NULL,
-                                            cbinfo?cbdispatch_step:NULL,
-                                            cbinfo?cbdispatch_final:NULL)
+                res=sqlite3_create_function_v2(self->db,
+					       name,
+					       numargs,
+					       SQLITE_UTF8,
+					       cbinfo,
+					       NULL,
+					       cbinfo?cbdispatch_step:NULL,
+					       cbinfo?cbdispatch_final:NULL,
+					       apsw_free_func)
                 );
 
-  if(callable==Py_None)
-    PyMem_Free(name);
-
   if(res)
     {
+      /* Note: On error sqlite3_create_function_v2 calls the
+	 destructor (apsw_free_func)! */
       SET_EXC(res, self->db);
       goto finally;
     }
 
-  if(callable!=Py_None)
-    /* put cbinfo into the list */
-    PyList_Append(self->functions, (PyObject*)cbinfo);
-    
+  if(callable==Py_None)
+    PyMem_Free(name);
 
  finally:
-  Py_XDECREF(cbinfo);
   if(PyErr_Occurred())
     return NULL;
   Py_RETURN_NONE;
@@ -2620,19 +2559,21 @@ Connection_createcollation(Connection *self, PyObject *args)
   Py_RETURN_NONE;
 }
 
-/** .. method:: filecontrol(op, pointer)
+/** .. method:: filecontrol(dbname, op, pointer) -> bool
 
   Calls the :meth:`~VFSFile.xFileControl` method on the :ref:`VFS`
   implementing :class:`file access <VFSFile>` for the database.
 
+  :param dbname: The name of the database to affect (eg "main", "temp", attached name)
   :param op: A `numeric code
     <http://sqlite.org/c3ref/c_fcntl_lockstate.html>`_ with values less
     than 100 reserved for SQLite internal use.
   :param pointer: A number which is treated as a ``void pointer`` at the C level.
 
-  The method does not return anything.  If you want data returned back
-  then the *pointer* needs to point to something mutable.  Here is an
-  example using `ctypes
+  :returns: True or False indicating if the VFS understood the op.
+
+  If you want data returned back then the *pointer* needs to point to
+  something mutable.  Here is an example using `ctypes
   <http://www.python.org/doc/2.5.2/lib/module-ctypes.html>`_ of
   passing a Python dictionary to :meth:`~VFSFile.xFileControl` which
   can then modify the dictionary to set return values::
@@ -2641,6 +2582,7 @@ Connection_createcollation(Connection *self, PyObject *args)
     objwrap=ctypes.py_object(obj)        # objwrap must live before and after the call else
                                          # it gets garbage collected
     connection.filecontrol(
+             "main",                     # which db
              123,                        # our op code
              ctypes.addressof(objwrap))  # get pointer
 
@@ -2652,9 +2594,17 @@ Connection_createcollation(Connection *self, PyObject *args)
             # play with obj - you can use id() to verify it is the same
             print obj["foo"]
             obj["result"]="it worked"
+	    return True
         else:
             # pass to parent/superclass
-            super(MyFile, self).xFileControl(op, pointer)
+            return super(MyFile, self).xFileControl(op, pointer)
+
+  This is how you set the chunk size by which the database grows.  Do
+  not combine it into one line as the c_int would be garbage collected
+  before the filecontrol call is made::
+
+     chunksize=ctypes.c_int(32768)
+     connection.filecontrol("main", apsw.SQLITE_FCNTL_CHUNK_SIZE, ctypes.addressof(chunksize))
 
   -* sqlite3_file_control
 */
@@ -2664,7 +2614,7 @@ Connection_filecontrol(Connection *self, PyObject *args)
 {
   PyObject *pyptr;
   void *ptr=NULL;
-  int res, op;
+  int res=SQLITE_ERROR, op;
   char *dbname=NULL;
 
   CHECK_USE(NULL);
@@ -2686,7 +2636,8 @@ Connection_filecontrol(Connection *self, PyObject *args)
 
   PYSQLITE_CON_CALL(res=sqlite3_file_control(self->db, dbname, op, ptr));
 
-  SET_EXC(res, self->db);
+  if(res!=SQLITE_OK && res!=SQLITE_NOTFOUND)
+    SET_EXC(res, self->db);
 
  finally:
   if(dbname) PyMem_Free(dbname);
@@ -2694,7 +2645,9 @@ Connection_filecontrol(Connection *self, PyObject *args)
   if(PyErr_Occurred())
     return NULL;
 
-  Py_RETURN_NONE;
+  if(res==SQLITE_NOTFOUND)
+    Py_RETURN_FALSE;
+  Py_RETURN_TRUE;
 }
 
 /** .. method:: sqlite3pointer() -> int
@@ -2753,33 +2706,46 @@ Connection_wal_autocheckpoint(Connection *self, PyObject *arg)
   return NULL;
 }
 
-/** .. method:: wal_checkpoint(dbname=None)
+/** .. method:: wal_checkpoint(dbname=None, mode=apsw.SQLITE_CHECKPOINT_PASSIVE) -> ( int, int )
 
-    Does an immediate checkpoint on all attached databases, or just **dbname** if specified.
+    Does a WAL checkpoint.  Has no effect if the database(s) are not in WAL mode.
 
-  -* sqlite3_wal_checkpoint
+    :param dbname:  The name of the database or all databases if None
+
+    :param mode: One of the `checkpoint modes <http://sqlite.org/c3ref/wal_checkpoint_v2.html>`__.
+
+    :return: A tuple of the size of the WAL log in frames and the
+       number of frames checkpointed as described in the
+       `documentation
+       <http://sqlite.org/c3ref/wal_checkpoint_v2.html>`__.
+
+  -* sqlite3_wal_checkpoint_v2
 */
 static PyObject *
-Connection_wal_checkpoint(Connection *self, PyObject *args)
+Connection_wal_checkpoint(Connection *self, PyObject *args, PyObject *kwargs)
 {
+  static char *kwlist[]={"dbname", "mode", NULL};
   int res;
   char *dbname=NULL;
+  int mode=SQLITE_CHECKPOINT_PASSIVE;
+  int nLog=0, nCkpt=0;
 
   CHECK_USE(NULL);
   CHECK_CLOSED(self, NULL);
 
-  if(!PyArg_ParseTuple(args, "|es:wal_checkpoint(dbname=None)", STRENCODING, &dbname))
+  if(!PyArg_ParseTupleAndKeywords(args, kwargs, "|esi:wal_checkpoint(dbname=None)", 
+				  kwlist, STRENCODING, &dbname, &mode))
     return NULL;
 
   APSW_FAULT_INJECT(WalCheckpointFails,
-		    PYSQLITE_CON_CALL(res=sqlite3_wal_checkpoint(self->db, dbname)),
+		    PYSQLITE_CON_CALL(res=sqlite3_wal_checkpoint_v2(self->db, dbname, mode, &nLog, &nCkpt)),
 		    res=SQLITE_IOERR);
 
   SET_EXC(res, self->db);
   PyMem_Free(dbname);
   /* done */
   if (res==SQLITE_OK)
-    Py_RETURN_NONE;
+    return Py_BuildValue("ii", nLog, nCkpt);
   return NULL;
 }
 
@@ -3163,6 +3129,54 @@ Connection_exit(Connection *self, PyObject *args)
   Py_RETURN_FALSE;
 }
 
+/** .. method:: config(op[, *args])
+
+    :param op: A `configuration operation
+      <http://sqlite.org/c3ref/c_dbconfig_enable_fkey.html>`__
+    :param args: Zero or more arguments as appropriate for *op*
+
+    -* sqlite3_db_config
+*/
+static PyObject *
+Connection_config(Connection *self, PyObject *args)
+{
+  long opt;
+  int res;
+
+  CHECK_USE(NULL);
+  CHECK_CLOSED(self, NULL);
+
+  if(PyTuple_GET_SIZE(args)<1 || !PyIntLong_Check(PyTuple_GET_ITEM(args, 0)))
+    return PyErr_Format(PyExc_TypeError, "There should be at least one argument with the first being a number");
+
+  opt=PyIntLong_AsLong(PyTuple_GET_ITEM(args,0));
+  if(PyErr_Occurred())
+    return NULL;
+
+  switch(opt)
+    {
+    case SQLITE_DBCONFIG_ENABLE_FKEY:
+    case SQLITE_DBCONFIG_ENABLE_TRIGGER:
+      {
+	int opdup, val, current;
+	if(!PyArg_ParseTuple(args, "ii", &opdup, &val))
+	  return NULL;
+
+	APSW_FAULT_INJECT(DBConfigFails,
+			  PYSQLITE_CON_CALL(res=sqlite3_db_config(self->db, opdup, val, &current)),
+			  res=SQLITE_NOMEM);
+	if(res!=SQLITE_OK)
+	  {
+	    SET_EXC(res, self->db);
+	    return NULL;
+	  }
+	return PyInt_FromLong(current);
+      }
+    default:
+      return PyErr_Format(PyExc_ValueError, "Unknown config operation %d", (int)opt);
+    }
+}
+
 /** .. attribute:: filename
 
   The filename used to open the database.
@@ -3263,8 +3277,10 @@ static PyMethodDef Connection_methods[] = {
    "Context manager exit"},
   {"wal_autocheckpoint", (PyCFunction)Connection_wal_autocheckpoint, METH_O,
    "Set wal checkpoint threshold"},
-  {"wal_checkpoint", (PyCFunction)Connection_wal_checkpoint, METH_VARARGS,
+  {"wal_checkpoint", (PyCFunction)Connection_wal_checkpoint, METH_VARARGS|METH_KEYWORDS,
    "Do immediate WAL checkpoint"},
+  {"config", (PyCFunction)Connection_config, METH_VARARGS,
+   "Configure this connection"},
   {0, 0, 0, 0}  /* Sentinel */
 };
 
