@@ -210,7 +210,8 @@ resetcursor(APSWCursor *self, int force)
   
   if(!force && self->status!=C_DONE && self->emiter)
     {
-      PyObject *next=PyIter_Next(self->emiter);
+      PyObject *next;
+      INUSE_CALL(next=PyIter_Next(self->emiter));
       if(next)
         {
           Py_DECREF(next);
@@ -365,10 +366,7 @@ APSWCursor_getdescription(APSWCursor *self)
   CHECK_CURSOR_CLOSED(NULL);
 
   if(!self->statement)
-    {
-      PyErr_Format(ExcComplete, "Can't get description for statements that have completed execution");
-      return NULL;
-    }
+    return PyErr_Format(ExcComplete, "Can't get description for statements that have completed execution");
   
   ncols=sqlite3_column_count(self->statement->vdbestatement);
   result=PyTuple_New(ncols);
@@ -781,7 +779,7 @@ APSWCursor_step(APSWCursor *self)
             }
 
           /* we are in executemany mode */
-          next=PyIter_Next(self->emiter);
+          INUSE_CALL(next=PyIter_Next(self->emiter));
           if(PyErr_Occurred())
             {
               assert(!next);
@@ -942,10 +940,7 @@ APSWCursor_execute(APSWCursor *self, PyObject *args)
   assert(PyTuple_Check(args));
 
   if(PyTuple_GET_SIZE(args)<1 || PyTuple_GET_SIZE(args)>2)
-    {
-      PyErr_Format(PyExc_TypeError, "Incorrect number of arguments.  execute(statements [,bindings])");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "Incorrect number of arguments.  execute(statements [,bindings])");
 
   query=PyTuple_GET_ITEM(args, 0);
   if (PyTuple_GET_SIZE(args)==2)
@@ -1058,12 +1053,9 @@ APSWCursor_executemany(APSWCursor *self, PyObject *args)
 
   self->emiter=PyObject_GetIter(theiterable);
   if (!self->emiter)
-    {
-      PyErr_Format(PyExc_TypeError, "2nd parameter must be iterable");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "2nd parameter must be iterable");
 
-  next=PyIter_Next(self->emiter);
+  INUSE_CALL(next=PyIter_Next(self->emiter));
   if(!next && PyErr_Occurred())
     return NULL;
   if(!next)
